<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Many check function for database fields.
 * Used by myMVC_BaseDB-derived classes.
 */
class myMVC_TypeCheck
{
  /**
   * Check function for typical autoincrement ID (INT(11) NOT NULL)
   */
  public static function checkAutoIncrementalId($value, $type)
  {
    if($value === NULL && $type == 'save')
      return true;

    return !(!is_numeric($value) || $value < -2147483648 || $value > 2147483647);
  }

  /**
   * Check function for Signed INT NOT NULL
   */
  public static function checkSignedIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < -2147483648 || $value > 2147483647);
  }

  /**
   * Check function for Unsigned INT NOT NULL
   */
  public static function checkUnsignedIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < 0 || $value > 4294967295);
  }

  /**
   * Check function for Signed INT NULL
   */
  public static function checkSignedIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < -2147483648 || $value > 2147483647);
  }

  /**
   * Check function for Unsigned INT NULL
   */
  public static function checkUnsignedIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < 0 || $value > 4294967295);
  }

  /**
   * Check function for Signed MEDIUMINT NOT NULL
   */
  public static function checkSignedMediumIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < -8388608 || $value > 8388607);
  }

  /**
   * Check function for Unsigned MEDIUMINT NOT NULL
   */
  public static function checkUnsignedMediumIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < 0 || $value > 16777215);
  }

  /**
   * Check function for Signed MEDIUMINT NULL
   */
  public static function checkSignedMediumIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < -8388608 || $value > 8388607);
  }

  /**
   * Check function for Unsigned MEDIUMINT NULL
   */
  public static function checkUnsignedMediumIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < 0 || $value > 16777215);
  }

  /**
   * Check function for Signed SMALLINT NOT NULL
   */
  public static function checkSignedSmallIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < -32768 || $value > 32767);
  }

  /**
   * Check function for Unsigned SMALLINT NOT NULL
   */
  public static function checkUnsignedSmallIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < 0 || $value > 65535);
  }

  /**
   * Check function for Signed SMALLINT NULL
   */
  public static function checkSignedSmallIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < -32768 || $value > 32767);
  }

  /**
   * Check function for Unsigned SMALLINT NULL
   */
  public static function checkUnsignedSmallIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < 0 || $value > 65535);
  }

  /**
   * Check function for Signed TINYINT NOT NULL
   */
  public static function checkSignedTinyIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < -128 || $value > 127);
  }

  /**
   * Check function for Unsigned TINYINT NOT NULL
   */
  public static function checkUnsignedTinyIntNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || $value < 0 || $value > 255);
  }

  /**
   * Check function for Signed TINYINT NULL
   */
  public static function checkSignedTinyIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < -128 || $value > 127);
  }

  /**
   * Check function for Unsigned TINYINT NULL
   */
  public static function checkUnsignedTinyIntNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || $value < 0 || $value > 255);
  }

  /**
   * Check function for boolean value NOT NULL
   */
  public static function checkBoolNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_numeric($value) || ($value != 0 && $value != 1));
  }

  /**
   * Check function for boolean value NULL
   */
  public static function checkBoolNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_numeric($value) || ($value != 0 && $value != 1));
  }

  /**
   * Check function for VARCHAR(255) NOT NULL
   */
  public static function checkString255NotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255);
  }

  /**
   * Check function for VARCHAR(255) NULL
   */
  public static function checkString255Null($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_string($value) || strlen($value) > 255);
  }

  /**
   * Check function for TEXT NOT NULL
   */
  public static function checkTextNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 65535);
  }

  /**
   * Check function for TEXT NULL
   */
  public static function checkTextNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_string($value) || strlen($value) > 65535);
  }

  /**
   * Check function for E-Mail address NOT NULL
   */
  public static function checkEMailNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    if($type == 'search')
    {
      if(!is_string($value))
        return false;
      $regexp = '/^[A-z0-9\-\.\@]*$/';
      return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
    }
    $regexp = '/^[A-z0-9][\w.-]*@[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}$/';
    return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
  }

  /**
   * Check function for E-Mail address NULL
   */
  public static function checkEMailNull($value, $type)
  {
    if($value === NULL || $value == '')
      return true;

    if($type == 'search')
    {
      if(!is_string($value))
        return false;
      $regexp = '/^[A-z0-9\-\.\@]*$/';
      return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
    }
    $regexp = '/^[A-z0-9][\w.-]*@[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}$/';
    return !(!is_string($value) || strlen($value) > 255 || preg_match($regexp, $value) == 0);
  }

  /**
   * Check function for URL NOT NULL
   */
  public static function checkURLNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    if($type == 'search')
    {
      if(!is_string($value))
        return false;
      $regexp = '/^[A-z0-9\-\.\:\/]*$/';
      return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
    }
    $regexp = '/^((http\:\/\/)|(https\:\/\/)|(ftp\:\/\/))?[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}(\/)?[A-z0-9\.\?\=\&\-\/]*$/';
    return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
  }

  /**
   * Check function for URL NULL
   */
  public static function checkURLNull($value, $type)
  {
    if($value === NULL || $value == '')
      return true;

    if($type == 'search')
    {
      if(!is_string($value))
        return false;
      $regexp = '/^[A-z0-9\-\.\:\/]*$/';
      return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
    }
    $regexp = '/^((http\:\/\/)|(https\:\/\/)|(ftp\:\/\/))?[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}(\/)?[A-z0-9\.\?\=\&\-\/]*$/';
    return !(!is_string($value) || strlen($value) > 255 || preg_match($regexp, $value) == 0);
  }

  /**
   * Check function for domain name NOT NULL
   */
  public static function checkDomainnameNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    if($type == 'search')
    {
      if(!is_string($value))
        return false;
      $regexp = '/^[A-z0-9\-\.]*$/';
      return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
    }
    $regexp = '/^[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}$/';
    return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
  }

  /**
   * Check function for domain name NULL
   */
  public static function checkDomainnameNull($value, $type)
  {
    if($value === NULL || $value == '')
      return true;

    if($type == 'search')
    {
      if(!is_string($value))
        return false;
      $regexp = '/^[A-z0-9\-\.]*$/';
      return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 255 || preg_match($regexp, $value) == 0);
    }
    $regexp = '/^[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}$/';
    return !(!is_string($value) || strlen($value) > 255 || preg_match($regexp, $value) == 0);
  }

  /**
   * Check function for date NOT NULL (Format: YYYY-MM-DD)
   */
  public static function checkDateNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{4})[\-]{1}([0-9]{2})[\-]{1}([0-9]{2})$/';
    if(preg_match($regexp, $value, $matches, PREG_OFFSET_CAPTURE) == 0)
      return false;
    if(!checkdate($matches[2][0], $matches[3][0], $matches[1][0]))
      return false;

    return true;
  }

  /**
   * Check function for date NULL (Format: YYYY-MM-DD)
   */
  public static function checkDateNull($value, $type)
  {
    if($value === NULL)
      return true;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{4})[\-]{1}([0-9]{2})[\-]{1}([0-9]{2})$/';
    if(preg_match($regexp, $value, $matches, PREG_OFFSET_CAPTURE) == 0)
      return false;
    if(!checkdate($matches[2][0], $matches[3][0], $matches[1][0]))
      return false;

    return true;
  }

  /**
   * Check function for time NOT NULL (Format: HH:MM:SS)
   */
  public static function checkTimeNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{2})[\:]{1}([0-9]{2})[\:]{1}([0-9]{2})$/';
    if(preg_match($regexp, $value, $matches, PREG_OFFSET_CAPTURE) == 0)
      return false;
    if($matches[1][0] > 23 ||
       $matches[2][0] > 59 ||
       $matches[3][0] > 59)
      return false;

    return true;
  }

  /**
   * Check function for time NULL (Format: HH:MM:SS)
   */
  public static function checkTimeNull($value, $type)
  {
    if($value === NULL)
      return true;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{2})[\:]{1}([0-9]{2})[\:]{1}([0-9]{2})$/';
    if(preg_match($regexp, $value, $matches, PREG_OFFSET_CAPTURE) == 0)
      return false;
    if($matches[1][0] > 23 ||
       $matches[2][0] > 59 ||
       $matches[3][0] > 59)
      return false;

    return true;
  }

  /**
   * Check function for date NOT NULL (Format: YYYY-MM-DD HH:MM:SS)
   */
  public static function checkDateTimeNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{4})[\-]{1}([0-9]{2})[\-]{1}([0-9]{2})[\ ]([0-9]{2})[\:]{1}([0-9]{2})[\:]{1}([0-9]{2})$/';
    if(preg_match($regexp, $value, $matches, PREG_OFFSET_CAPTURE) == 0)
      return false;
    if(!checkdate($matches[2][0], $matches[3][0], $matches[1][0]))
      return false;
    if($matches[4][0] > 23 ||
       $matches[5][0] > 59 ||
       $matches[6][0] > 59)
      return false;

    return true;
  }

  /**
   * Check function for date NULL (Format: YYYY-MM-DD HH:MM:SS)
   */
  public static function checkDateTimeNull($value, $type)
  {
    if($value === NULL)
      return true;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{4})[\-]{1}([0-9]{2})[\-]{1}([0-9]{2})[\ ]([0-9]{2})[\:]{1}([0-9]{2})[\:]{1}([0-9]{2})$/';
    if(preg_match($regexp, $value, $matches, PREG_OFFSET_CAPTURE) == 0)
      return false;
    if(!checkdate($matches[2][0], $matches[3][0], $matches[1][0]))
      return false;
    if($matches[4][0] > 23 ||
       $matches[5][0] > 59 ||
       $matches[6][0] > 59)
      return false;

    return true;
  }

  /**
   * Check function for IPv4 addresses (NOT NULL)
   */
  public static function checkIPNotNull($value, $type)
  {
    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{1,3}\.){3}[0]{1}$/';
    return !($type == 'save' && (ip2long($value) === false || preg_match($regexp, strtolower($value)) != 0));
  }

  /**
   * Check function for IPv4 addresses (NULL)
   */
  public static function checkIPNull($value, $type)
  {
    if($value === NULL)
      return true;

    if(!is_string($value))
      return false;
    $regexp = '/^([0-9]{1,3}\.){3}[0]{1}$/';
    return !($type == 'save' && (ip2long($value) === false || preg_match($regexp, strtolower($value)) != 0));
  }

  /**
   * Check function for MAC addresses (NOT NULL)
   */
  public static function checkMacNotNull($value, $type)
  {
    if(!is_string($value) || strlen($value) > 17)
      return false;
    $regexp = '/^([a-f0-9]{2}\:){5}[a-f0-9]{2}$/';
    return !($type == 'save' && preg_match($regexp, strtolower($value)) == 0);
  }

  /**
   * Check function for MAC addresses (NULL)
   */
  public static function checkMacNull($value, $type)
  {
    if($value === NULL)
      return true;

    if(!is_string($value) || strlen($value) > 17)
      return false;
    $regexp = '/^([a-f0-9]{2}\:){5}[a-f0-9]{2}$/';
    return !($type == 'save' && preg_match($regexp, strtolower($value)) == 0);
  }

  /**
   * Check function for IPv4 network (NOT NULL)
   */
  public static function checkNetworkNotNull($value, $type)
  {
    if(ip2long($value) === false)
      return false;
    $regexp = '/^([0-1]{31})[0]{1}$/';
    return !(preg_match($regexp, sprintf("%032b", ip2long($value))) == 0);
  }

  /**
   * Check function for IPv4 network (NULL)
   */
  public static function checkNetworkNull($value, $type)
  {
    if($value === NULL)
      return true;

    if(ip2long($value) === false)
      return false;
    $regexp = '/^([0-1]{31})[0]{1}$/';
    return !(preg_match($regexp, sprintf("%032b", ip2long($value))) == 0);
  }

  /**
   * Check function for IPv4 netmask (NOT NULL)
   */
  public static function checkMaskNotNull($value, $type)
  {
    $regexp = '/^(1){1}([0-1]{30})[0]?$/';
    return !(preg_match($regexp, sprintf("%032b", ip2long($value))) == 0);
  }

  /**
   * Check function for IPv4 netmask (NULL)
   */
  public static function checkMaskNull($value, $type)
  {
    if($value === NULL)
      return true;

    $regexp = '/^(1){1}([0-1]{30})[0]?$/';
    return !(preg_match($regexp, sprintf("%032b", ip2long($value))) == 0);
  }
}
?>
